using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace StablePoint.Protocols.ThingSpeak {
    public static class ThingSpeak {
        private const string BaseUrl = "http://api.thingspeak.com/";

        /// <summary>
        /// Sends field data to the Thingspeak, using provided API key.
        /// </summary>
        /// <param name="channelKey">Write API key that you have to find on your Thingspeak channel website. Without it, you cannot send any data.</param>
        /// <param name="field1">Field 1 of the channel.</param>
        /// <param name="field2">Field 2 of the channel.</param>
        /// <param name="field3">Field 3 of the channel.</param>
        /// <param name="field4">Field 4 of the channel.</param>
        /// <param name="field5">Field 5 of the channel.</param>
        /// <param name="field6">Field 6 of the channel.</param>
        /// <param name="field7">Field 7 of the channel.</param>
        /// <param name="field8">Field 8 of the channel.</param>
        /// <returns></returns>
        public static int PostFeed(string channelKey, string field1 = "", string field2 = "", string field3 = "", string field4 = "", string field5 = "", string field6 = "", string field7 = "", string field8 = "") {
            var feedUrl = "update?key=" + channelKey;

            if (field1 != string.Empty) feedUrl += "&field1=" + field1;
            if (field2 != string.Empty) feedUrl += "&field2=" + field2;
            if (field3 != string.Empty) feedUrl += "&field3=" + field3;
            if (field4 != string.Empty) feedUrl += "&field4=" + field4;
            if (field5 != string.Empty) feedUrl += "&field5=" + field5;
            if (field6 != string.Empty) feedUrl += "&field6=" + field6;
            if (field7 != string.Empty) feedUrl += "&field7=" + field7;
            if (field8 != string.Empty) feedUrl += "&field8=" + field8;

            var uri = BaseUrl + feedUrl;
            var request = (HttpWebRequest)WebRequest.Create(uri);
            var response = request.GetResponseAsync().Result;
            string serverResponse = "";

            try {
                serverResponse = ReadStreamAsString(response.GetResponseStream()).Result;
            } catch (Exception ex) {
                //throw new ApplicationException("An network-related error occured in the ThingSpeak API Post method.  Please check the inner exception", ex);
            }

            try {
                return int.Parse(serverResponse);
            } catch (Exception) {
                throw new IOException("The ThingSpeak server returned an error message : " + serverResponse);
            }
        }

        public static int PostFeed(string channelKey, int fieldNumber, string fieldValue) {
            switch (fieldNumber) {
                case 1:
                    return PostFeed(channelKey, fieldValue);

                case 2:
                    return PostFeed(channelKey, "", fieldValue);
                case 3:
                    return PostFeed(channelKey, "", "", fieldValue);
                case 4:
                    return PostFeed(channelKey, "", "", "", fieldValue);
                case 5:
                    return PostFeed(channelKey, "", "", "", "", fieldValue);
                case 6:
                    return PostFeed(channelKey, "", "", "", "", "", fieldValue);
                case 7:
                    return PostFeed(channelKey, "", "", "", "", "", "", fieldValue);
                case 8:
                    return PostFeed(channelKey, "", "", "", "", "", "", "", fieldValue);

                default:
                    return PostFeed(channelKey, "", "", "", "", "", "", "", fieldValue);
            }
        }

        public static string RetrieveTalkBackCommand(string talkBackId, string apiKey) {
            var request = (HttpWebRequest)WebRequest.Create(BaseUrl + "talkbacks/" + talkBackId + "/commands/execute?api_key=" + apiKey);
            var response = request.GetResponseAsync().Result;

            try {
                var buffer = new byte[160];
                var position = 0;
                int bytesRead;
                var stream = response.GetResponseStream();

                do {
                    Task.Delay(TimeSpan.FromMilliseconds(10)).Wait();
                    bytesRead = stream.Read(buffer, position, buffer.Length - position);
                    position += bytesRead;
                } while (bytesRead > 0);

                if (position > 0) return new string(Encoding.UTF8.GetChars(buffer));
                else return "";
            } catch (Exception ex) {
                throw new IOException("An network-related error occured in the ThingSpeak API Post method.  Please check the inner exception", ex);
            }
        }

        private static async Task<string> ReadStreamAsString(Stream stream) {
            var buffer = new byte[128];
            var position = 0;
            int bytesRead;

            do {
                await Task.Delay(TimeSpan.FromMilliseconds(10));
                bytesRead = stream.Read(buffer, position, buffer.Length - position);
                position += bytesRead;
            } while (bytesRead > 0);

            return new string(Encoding.UTF8.GetChars(buffer));
        }
    }
}